# SPDX-FileCopyrightText: Copyright © 2023 Idiap Research Institute <contact@idiap.ch>
#
# SPDX-License-Identifier: GPL-3.0-or-later
"""Data loading code."""

import pathlib

import numpy
import PIL.Image


def remove_black_borders(
    img: PIL.Image.Image, threshold: int = 0
) -> PIL.Image.Image:
    """Remove black borders of CXR.

    Parameters
    ----------
        img
            A PIL image
        threshold
            Threshold value from which borders are considered black.
            Defaults to 0.

    Returns
    -------
        A PIL image with black borders removed
    """

    img_array = numpy.asarray(img)

    if len(img_array.shape) == 2:  # single channel
        mask = numpy.asarray(img_array) > threshold
        return PIL.Image.fromarray(
            img_array[numpy.ix_(mask.any(1), mask.any(0))]
        )

    elif len(img_array.shape) == 3 and img_array.shape[2] == 3:
        r_mask = img_array[:, :, 0] > threshold
        g_mask = img_array[:, :, 1] > threshold
        b_mask = img_array[:, :, 2] > threshold

        mask = r_mask | g_mask | b_mask
        return PIL.Image.fromarray(
            img_array[numpy.ix_(mask.any(1), mask.any(0))]
        )

    else:
        raise NotImplementedError


class RemoveBlackBorders:
    """Remove black borders of CXR."""

    def __init__(self, threshold=0):
        self.threshold = threshold

    def __call__(self, img):
        return remove_black_borders(img, self.threshold)


def load_pil(path: str | pathlib.Path) -> PIL.Image.Image:
    """Loads a sample data.

    Parameters
    ----------

    path
        The full path leading to the image to be loaded


    Returns
    -------

    image
        A PIL image
    """
    return PIL.Image.open(path)


def load_pil_grayscale(path: str | pathlib.Path) -> PIL.Image.Image:
    """Loads a sample data in grayscale mode ("L").

    Parameters
    ----------

    path
        The full path leading to the image to be loaded


    Returns
    -------

    image
        A PIL image in grayscale mode
    """
    return load_pil(path).convert("L")


def load_pil_rgb(path: str | pathlib.Path) -> PIL.Image.Image:
    """Loads a sample data in RGB mode ("RGB").

    Parameters
    ----------

    path
        The full path leading to the image to be loaded


    Returns
    -------

    image
        A PIL image in RGB mode
    """
    return load_pil(path).convert("RGB")
